package com.dimple.job.service.mapper;


import com.dimple.job.service.entity.SysJob;

import java.util.List;

/**
 * 调度任务信息 数据层
 *
 * @author Dimple
 */
public interface SysJobMapper {
    /**
     * 查询调度任务日志集合
     *
     * @param job 调度信息
     * @return 操作日志集合
     */
    List<SysJob> selectJobList(SysJob job);

    /**
     * 查询所有调度任务
     *
     * @return 调度任务列表
     */
    List<SysJob> selectJobAll();

    /**
     * 通过调度ID查询调度任务信息
     *
     * @param id 调度ID
     * @return 角色对象信息
     */
    SysJob selectJobById(Long id);

    /**
     * 通过调度ID删除调度任务信息
     *
     * @param id 调度ID
     * @return affected lines
     */
    int deleteJobById(Long id);

    /**
     * 批量删除调度任务信息
     *
     * @param ids 需要删除的数据ID
     * @return affected lines
     */
    int deleteJobByIds(Long[] ids);

    /**
     * 修改调度任务信息
     *
     * @param job 调度任务信息
     * @return affected lines
     */
    int updateJob(SysJob job);

    /**
     * 新增调度任务信息
     *
     * @param job 调度任务信息
     * @return affected lines
     */
    int insertJob(SysJob job);
}
